@@ -1,5 +1,5 @@
 package Net::Async::Webservice::UPS;
-$Net::Async::Webservice::UPS::VERSION = '1.0.7';
+$Net::Async::Webservice::UPS::VERSION = '1.1.0';
 {
   $Net::Async::Webservice::UPS::DIST = 'Net-Async-Webservice-UPS';
 }
@@ -23,8 +23,7 @@ use Net::Async::Webservice::UPS::Response::Rate;
 use Net::Async::Webservice::UPS::Response::Address;
 use Net::Async::Webservice::UPS::Response::ShipmentConfirm;
 use Net::Async::Webservice::UPS::Response::ShipmentAccept;
-use Net::Async::Webservice::UPS::Response::PackageResult;
-use Net::Async::Webservice::UPS::Response::Image;
+use Net::Async::Webservice::UPS::Response::QV;
 use MIME::Base64;
 use Future;
 use 5.010;
@@ -289,11 +288,7 @@ sub request_rate {
                     # TODO check this pairwise
                     rates => [ pairwise {
                         Net::Async::Webservice::UPS::Rate->new({
-                            billing_weight  => $a->{BillingWeight}{Weight},
-                            unit            => $a->{BillingWeight}{UnitOfMeasurement}{Code},
-                            total_charges   => $a->{TotalCharges}{MonetaryValue},
-                            total_charges_currency => $a->{TotalCharges}{CurrencyCode},
-                            weight          => $a->{Weight},
+                            %$a,
                             rated_package   => $b,
                             from            => $args->{from},
                             to              => $args->{to},
@@ -307,9 +302,8 @@ sub request_rate {
             @services = sort { $a->total_charges <=> $b->total_charges } @services;
 
             my $ret = Net::Async::Webservice::UPS::Response::Rate->new({
-                customer_context => $response->{Response}{TransactionReference}{CustomerContext},
+                %$response,
                 services => \@services,
-                ( $response->{Error} ? (warnings => $response->{Error}) : () ),
             });
 
             $self->cache->set($cache_key,$ret) if $self->does_caching;
@@ -365,21 +359,15 @@ sub validate_address {
             for my $address (@{$response->{AddressValidationResult}}) {
                 next if $address->{Quality} < (1 - $tolerance);
                 for my $possible_postal_code ($address->{PostalCodeLowEnd} .. $address->{PostalCodeHighEnd}) {
-                    push @addresses, Net::Async::Webservice::UPS::Address->new({
-                        quality         => $address->{Quality},
-                        postal_code     => $possible_postal_code,
-                        city            => $address->{Address}->{City},
-                        state           => $address->{Address}->{StateProvinceCode},
-                        country_code    => "US",
-                    });
+                    $address->{Address}{PostalCode} = $possible_postal_code;
+                    push @addresses, Net::Async::Webservice::UPS::Address->new($address);
                 }
             }
 
 
             my $ret = Net::Async::Webservice::UPS::Response::Address->new({
-                customer_context => $response->{Response}{TransactionReference}{CustomerContext},
+                %$response,
                 addresses => \@addresses,
-                ( $response->{Error} ? (warnings => $response->{Error}) : () ),
             });
 
             $self->cache->set($cache_key,$ret) if $self->does_caching;
@@ -454,24 +442,14 @@ sub validate_street_address {
             my @addresses;
             for my $ak (@{$response->{AddressKeyFormat}}) {
                 push @addresses, Net::Async::Webservice::UPS::Address->new({
-                    quality => $quality,
-                    building_name => $ak->{BuildingName},
-                    address => $ak->{AddressLine}->[0],
-                    address2 => $ak->{AddressLine}->[1],
-                    address3 => $ak->{AddressLine}->[2],
-                    postal_code => $ak->{PostcodePrimaryLow},
-                    postal_code_extended => $ak->{PostcodeExtendedLow},
-                    city => $ak->{PoliticalDivision2},
-                    state => $ak->{PoliticalDivision1},
-                    country_code => $ak->{CountryCode},
-                    is_residential => ( $response->{AddressClassification}->{Code} eq "2" ) ? 1 : 0,
+                    AddressKeyFormat => $ak,
+                    Quality => $quality,
                 });
             }
 
             my $ret = Net::Async::Webservice::UPS::Response::Address->new({
-                customer_context => $response->{Response}{TransactionReference}{CustomerContext},
+                %$response,
                 addresses => \@addresses,
-                ( $response->{Error} ? (warnings => $response->{Error}) : () ),
             });
 
             $self->cache->set($cache_key,$ret) if $self->does_caching;
@@ -545,19 +523,8 @@ sub ship_confirm {
         done => sub {
             my ($response) = @_;
 
-            my $weight = $response->{BillingWeight};
-            my $charges = $response->{ShipmentCharges};
-
             return Net::Async::Webservice::UPS::Response::ShipmentConfirm->new({
-                customer_context => $response->{Response}{TransactionReference}{CustomerContext},
-                unit => $weight->{UnitOfMeasurement}{Code},
-                billing_weight => $weight->{Weight},
-                currency => $charges->{TotalCharges}{CurrencyCode},
-                service_option_charges => $charges->{ServiceOptionsCharges}{MonetaryValue},
-                transportation_charges => $charges->{TransportationCharges}{MonetaryValue},
-                total_charges => $charges->{TotalCharges}{MonetaryValue},
-                shipment_digest => $response->{ShipmentDigest},
-                shipment_identification_number => $response->{ShipmentIdentificationNumber},
+                %$response,
                 packages => $packages,
             });
         },
@@ -565,21 +532,6 @@ sub ship_confirm {
 }
 
 
-sub _img_if($$) {
-    return ( $_[0] => Net::Async::Webservice::UPS::Response::Image->from_hash($_[1]) ) if $_[1] && %{$_[1]};
-    return;
-}
-
-sub _pair_if($$) {
-    return @_ if $_[1];
-    return;
-}
-
-sub _base64_if($$) {
-    return ($_[0],decode_base64($_[1])) if $_[1];
-    return;
-}
-
 sub ship_accept {
     state $argcheck = compile( Object, Dict[
         confirm => ShipmentConfirm,
@@ -609,51 +561,49 @@ sub ship_accept {
         done => sub {
             my ($response) = @_;
 
-            my $results = $response->{ShipmentResults};
-
-            my $weight = $results->{BillingWeight};
-            my $charges = $results->{ShipmentCharges};
-
             return Net::Async::Webservice::UPS::Response::ShipmentAccept->new({
-                customer_context => $response->{Response}{TransactionReference}{CustomerContext},
-                unit => $weight->{UnitOfMeasurement}{Code},
-                billing_weight => $weight->{Weight},
-                currency => $charges->{TotalCharges}{CurrencyCode},
-                service_option_charges => $charges->{ServiceOptionsCharges}{MonetaryValue},
-                transportation_charges => $charges->{TransportationCharges}{MonetaryValue},
-                total_charges => $charges->{TotalCharges}{MonetaryValue},
-                shipment_identification_number => $results->{ShipmentIdentificationNumber},
-                _pair_if( pickup_request_number => $results->{PickupRequestNumber} ),
-                _img_if( control_log => $results->{ControlLogReceipt} ),
-                package_results => [ pairwise {
-                    my ($pr,$pack) = ($a, $b);
-
-                    Net::Async::Webservice::UPS::Response::PackageResult->new({
-                        package => $pack,
-                        tracking_number => $pr->{TrackingNumber},
-                        currency => $pr->{ServiceOptionsCharges}{CurrencyCode},
-                        service_option_charges => $pr->{ServiceOptionsCharges}{MonetaryValue},
-                        _img_if( label => $pr->{LabelImage} ),
-                        ( $pr->{LabelImage}{InternationalSignatureGraphicImage} ?
-                          ( signature => Net::Async::Webservice::UPS::Response::Image->new({
-                              format => $pr->{LabelImage}{ImageFormat}{Code},
-                              base64_data => $pr->{LabelImage}{InternationalSignatureGraphicImage},
-                          }) ) : () ),
-                        _base64_if( html => $pr->{LabelImage}{HTMLImage} ),
-                        _base64_if( pdf417 => $pr->{LabelImage}{PDF417} ),
-                        _img_if( receipt => $pr->{Receipt}{Image} ),
-                        _img_if( form_image => $pr->{Form} ),
-                        _pair_if( form_code => $pr->{Form}{Code} ),
-                        _pair_if( form_group_id => $pr->{FormGroupId} ),
-                        _img_if( cod_turn_in => $pr->{CODTurnInPage} ),
-                    });
-                } @{$results->{PackageResults}//[]},@$packages ],
+                packages => $packages,
+                %$response,
             });
         },
     );
 }
 
 
+sub qv_events {
+    state $argcheck = compile( Object, Dict[
+        subscriptions => Optional[ArrayRef[QVSubscription]],
+        bookmark => Optional[Str],
+        customer_context => Optional[Str],
+    ]);
+
+    my ($self,$args) = $argcheck->(@_);
+
+    my %data = (
+        QuantumViewRequest => {
+            Request => {
+                TransactionReference => $self->transaction_reference($args),
+                RequestAction => 'QVEvents',
+            },
+            ( $args->{subscriptions} ? ( SubscriptionRequest => [
+                map { $_->as_hash } @{$args->{subscriptions}}
+            ] ) : () ),
+            ($args->{bookmark} ? (Bookmark => $args->{bookmark}) : () ),
+        },
+    );
+
+    $self->xml_request({
+        data => \%data,
+        url_suffix => '/QVEvents',
+    })->transform(
+        done => sub {
+            my ($response) = @_;
+            return Net::Async::Webservice::UPS::Response::QV->new($response);
+        }
+    );
+}
+
+
 sub xml_request {
     state $argcheck = compile(
         Object,
@@ -762,7 +712,7 @@ Net::Async::Webservice::UPS - UPS API client, non-blocking
 
 =head1 VERSION
 
-version 1.0.7
+version 1.1.0
 
 =head1 SYNOPSIS
 
@@ -1052,13 +1002,13 @@ Identical requests can be cached.
 
 =head2 C<ship_confirm>
 
-  my $shipconfirm_response = $usp->ship_confirm({
+  $ups->ship_confirm({
      from => $source_contact,
      to => $destination_contact,
      description => 'something',
      payment => $payment_method,
      packages => \@packages,
-  });
+  }) ==> $shipconfirm_response
 
 Performs a C<ShipConfirm> request to UPS. The parameters are:
 
@@ -1110,7 +1060,7 @@ optional string for reference purposes
 
 =back
 
-Returns an instance of
+Returns a L<Future> yielding an instance of
 L<Net::Async::Webservice::UPS::Response::ShipmentConfirm>.
 
 B<NOTE>: the API of this call may change in the future, let me know if
@@ -1118,9 +1068,9 @@ features you need are missing or badly understood!
 
 =head2 C<ship_accept>
 
-  my $shipaccept_response = $usp->ship_accept({
+  $ups->ship_accept({
       confirm => $shipconfirm_response,
-  });
+  }) ==> $shipaccept_response
 
 Performs a C<ShipAccept> request to UPS. The parameters are:
 
@@ -1136,9 +1086,38 @@ optional string for reference purposes
 
 =back
 
-Returns an instance of
+Returns a L<Future> yielding an instance of
 L<Net::Async::Webservice::UPS::Response::ShipmentAccept>.
 
+=head2 C<qv_events>
+
+  $ups->qv_events({
+      subscriptions => [ Net::Async::Webservice::UPS::QVSubscription->new(
+        name => 'MySubscription',
+      ) ],
+  }) ==> $qv_response
+
+Performs a C<QVEvennts> request to UPS. The parameters are:
+
+=over 4
+
+=item C<subscriptions>
+
+optional, array of L<Net::Async::Webservice::UPS::QVSubscription>, specifying what you want to retrieve
+
+=item C<bookmark>
+
+optional, string retrieved from a previous call, used for pagination (see L<Net::Async::Webservice::UPS::Response::QV>)
+
+=item C<customer_context>
+
+optional string for reference purposes
+
+=back
+
+Returns a L<Future> yielding an instance of
+L<Net::Async::Webservice::UPS::Response::QV>.
+
 =head2 C<xml_request>
 
   $ups->xml_request({
