@@ -1,5 +1,5 @@
 package Net::Async::Webservice::UPS::Address;
-$Net::Async::Webservice::UPS::Address::VERSION = '1.0.7';
+$Net::Async::Webservice::UPS::Address::VERSION = '1.1.0';
 {
   $Net::Async::Webservice::UPS::Address::DIST = 'Net-Async-Webservice-UPS';
 }
@@ -7,6 +7,7 @@ use Moo;
 use 5.010;
 use Types::Standard qw(Str Int Bool StrictNum);
 use Net::Async::Webservice::UPS::Types ':types';
+use Net::Async::Webservice::UPS::Response::Utils ':all';
 use namespace::autoclean;
 
 # ABSTRACT: an address for UPS
@@ -22,7 +23,7 @@ has city => (
 has postal_code => (
     is => 'ro',
     isa => Str,
-    required => 1,
+    required => 0,
 );
 
 
@@ -32,20 +33,6 @@ has postal_code_extended => (
     required => 0,
 );
 
-around BUILDARGS => sub {
-    my ($orig,$class,@etc) = @_;
-    my $args = $class->$orig(@etc);
-    if ($args->{postal_code}
-            and not defined $args->{postal_code_extended}
-                and $args->{postal_code} =~ m{\A(\d+)-(\d+)\z}) {
-        $args->{postal_code} = $1;
-        $args->{postal_code_extended} = $2;
-    }
-    my @undef_k = grep {not defined $args->{$_} } keys %$args;
-    delete @$args{@undef_k};
-    return $args;
-};
-
 
 has state => (
     is => 'ro',
@@ -150,13 +137,15 @@ sub as_hash {
     my ($self, $shape) = @_;
     $shape //= 'AV';
 
+    set_implied_argument($self);
+
     if ($shape eq 'AV') {
         return {
             Address => {
                 CountryCode => $self->country_code || "US",
                 PostalCode  => $self->postal_code,
-                ( $self->city ? ( City => $self->city) : () ),
-                ( $self->state ? ( StateProvinceCode => $self->state) : () ),
+                out_if(City=>'city'),
+                out_if(StateProvinceCode=>'state'),
                 ( $self->is_residential ? ( ResidentialAddressIndicator => undef ) : () ),
             }
         };
@@ -166,16 +155,16 @@ sub as_hash {
             AddressKeyFormat => {
                 CountryCode => $self->country_code || "US",
                 PostcodePrimaryLow  => $self->postal_code,
-                ( $self->postal_code_extended ? ( PostcodeExtendedLow => $self->postal_code_extended ) : () ),
-                ( $self->name ? ( ConsigneeName => $self->name ) : () ),
-                ( $self->building_name ? ( BuildingName => $self->building_name ) : () ),
+                out_if(PostcodeExtendedLow=>'postal_code_extended'),
+                out_if(ConsigneeName=>'name'),
+                out_if(BuildingName=>'building_name'),
                 AddressLine  => [
                     ( $self->address ? $self->address : () ),
                     ( $self->address2 ? $self->address2 : () ),
                     ( $self->address3 ? $self->address3 : () ),
                 ],
-                ( $self->state ? ( PoliticalDivision1 => $self->state ) : () ),
-                ( $self->city ? ( PoliticalDivision2 => $self->city ) : () ),
+                out_if(PoliticalDivision1=>'state'),
+                out_if(PoliticalDivision2=>'city'),
             }
         }
     }
@@ -184,19 +173,73 @@ sub as_hash {
             Address => {
                 CountryCode => $self->country_code || "US",
                 PostalCode  => $self->postal_code,
-                ( $self->address ? ( AddressLine1 => $self->address ) : () ),
-                ( $self->address2 ? ( AddressLine3 => $self->address2 ) : () ),
-                ( $self->address3 ? ( AddressLine3 => $self->address3 ) : () ),
-                ( $self->city ? ( City => $self->city) : () ),
-                ( $self->state ? ( StateProvinceCode => $self->state) : () ),
+                out_if(AddressLine1=>'address'),
+                out_if(AddressLine2=>'address2'),
+                out_if(AddressLine3=>'address3'),
+                out_if(City=>'city'),
+                out_if(StateProvinceCode=>'state'),
             }
         }
     }
     else {
-        die "bad address to_hash shape $shape";
+        die "bad address as_hash shape $shape";
     }
 }
 
+sub BUILDARGS {
+    my ($class,@etc) = @_;
+    my $hashref = $class->next::method(@etc);
+
+    my $data = $hashref->{Address} || $hashref->{AddressKeyFormat} || $hashref->{AddressArtifactFormat};
+
+    if (not $data) {
+        if ($hashref->{postal_code}
+                and not defined $hashref->{postal_code_extended}
+                    and $hashref->{postal_code} =~ m{\A(\d+)-(\d+)\z}) {
+            $hashref->{postal_code} = $1;
+            $hashref->{postal_code_extended} = $2;
+        }
+        my @undef_k = grep {not defined $hashref->{$_} } keys %$hashref;
+        delete @$hashref{@undef_k};
+        return $hashref;
+    }
+
+    set_implied_argument($data);
+
+    return {
+        country_code => 'US', # default,
+        pair_if(quality=>$hashref->{Quality}),
+        in_if(country_code=>'CountryCode'),
+        in_if(postal_code=>'PostalCode'),
+        in_if(postal_code=>'PostcodePrimaryLow'),
+        in_if(city=>'City'),
+        in_if(city=>'PoliticalDivision2'),
+        in_if(state=>'StateProvinceCode'),
+        in_if(state=>'PoliticalDivision1'),
+        in_if(postal_code_extended=>'PostcodeExtendedLow'),
+        in_if(name=>'ConsigneeName'),
+        in_if(building_name=>'BuildingName'),
+        in_if(address=>'AddressLine1'),
+        in_if(address2=>'AddressLine2'),
+        in_if(address3=>'AddressLine3'),
+
+        ( exists $data->{ResidentialAddressIndicator} ? ( is_residential => 1 ) : () ),
+        ( exists $data->{AddressClassification} ? ( is_residential => $data->{AddressClassification}{Code} eq 2 ? 1 : 0 ) : () ),
+
+        ( $data->{StreetName} || $data->{StreetNumberLow} || $data->{StreetType} ? (
+            address => join(
+                ' ',grep { defined }
+                    @{$data}{qw(StreetNumberLow StreetName {StreetType)})
+                ) : () ),
+
+        ( ref($data->{AddressLine}) eq 'ARRAY' ? (
+            ( $data->{AddressLine}[0] ? ( address => $data->{AddressLine}[0] ) : () ),
+            ( $data->{AddressLine}[1] ? ( address2 => $data->{AddressLine}[1] ) : () ),
+            ( $data->{AddressLine}[2] ? ( address3 => $data->{AddressLine}[2] ) : () ),
+        ) : () ),
+    };
+}
+
 
 sub cache_id {
     my ($self) = @_;
@@ -227,7 +270,7 @@ Net::Async::Webservice::UPS::Address - an address for UPS
 
 =head1 VERSION
 
-version 1.0.7
+version 1.1.0
 
 =head1 ATTRIBUTES
 
@@ -237,7 +280,7 @@ String with the name of the city, optional.
 
 =head2 C<postal_code>
 
-String with the post code of the address, required.
+String with the post code of the address, usually required.
 
 =head2 C<postal_code_extended>
 
