@@ -1,11 +1,12 @@
 package Net::Async::Webservice::UPS::Response::PackageResult;
-$Net::Async::Webservice::UPS::Response::PackageResult::VERSION = '1.0.7';
+$Net::Async::Webservice::UPS::Response::PackageResult::VERSION = '1.1.0';
 {
   $Net::Async::Webservice::UPS::Response::PackageResult::DIST = 'Net-Async-Webservice-UPS';
 }
 use Moo;
 use Types::Standard qw(Str);
 use Net::Async::Webservice::UPS::Types qw(:types);
+use Net::Async::Webservice::UPS::Response::Utils ':all';
 use namespace::autoclean;
 
 # ABSTRACT: information about a package in a booked shipment
@@ -101,6 +102,35 @@ has package => (
     required => 1,
 );
 
+sub BUILDARGS {
+    my ($class,@etc) = @_;
+    my $ret = $class->next::method(@etc);
+
+    if ($ret->{TrackingNumber}) {
+        set_implied_argument($ret);
+        return {
+            package => $ret->{package},
+            tracking_number => $ret->{TrackingNumber},
+            currency => $ret->{ServiceOptionsCharges}{CurrencyCode},
+            service_option_charges => $ret->{ServiceOptionsCharges}{MonetaryValue},
+            img_if( label => $ret->{LabelImage} ),
+            ( $ret->{LabelImage}{InternationalSignatureGraphicImage} ?
+                  ( signature => Net::Async::Webservice::UPS::Response::Image->new({
+                      format => $ret->{LabelImage}{ImageFormat}{Code},
+                      base64_data => $ret->{LabelImage}{InternationalSignatureGraphicImage},
+                  }) ) : () ),
+            base64_if( html => $ret->{LabelImage}{HTMLImage} ),
+            base64_if( pdf417 => $ret->{LabelImage}{PDF417} ),
+            img_if( receipt => $ret->{Receipt}{Image} ),
+            img_if( form_image => $ret->{Form} ),
+            pair_if( form_code => $ret->{Form}{Code} ),
+            pair_if( form_group_id => $ret->{FormGroupId} ),
+            img_if( cod_turn_in => $ret->{CODTurnInPage} ),
+        };
+    }
+    return $ret;
+}
+
 1;
 
 __END__
@@ -115,7 +145,7 @@ Net::Async::Webservice::UPS::Response::PackageResult - information about a packa
 
 =head1 VERSION
 
-version 1.0.7
+version 1.1.0
 
 =head1 DESCRIPTION
 
@@ -184,6 +214,8 @@ Reference to the package given to the
 L<Net::Async::Webservice::UPS/ship_confirm> request, to which this
 result element refers to.
 
+=for Pod::Coverage BUILDARGS
+
 =head1 AUTHORS
 
 =over 4
