@@ -1,5 +1,5 @@
 package Net::Async::Webservice::UPS::Types;
-$Net::Async::Webservice::UPS::Types::VERSION = '1.0.7';
+$Net::Async::Webservice::UPS::Types::VERSION = '1.1.0';
 {
   $Net::Async::Webservice::UPS::Types::DIST = 'Net-Async-Webservice-UPS';
 }
@@ -20,6 +20,8 @@ use Type::Library
                     Tolerance Payment ImageType Image
                     Contact Shipper CreditCard Label
                     ShipmentConfirm PackageResult
+                    QVSubscription QVEvent QVFile QVDelivery QVGeneric
+                    QVOrigin QVManifest QVPackage QVException QVReference
               );
 use Type::Utils -all;
 use Types::Standard -types;
@@ -287,6 +289,18 @@ RateList->coercion->freeze;
 
 class_type ShipmentConfirm, { class => 'Net::Async::Webservice::UPS::Response::ShipmentConfirm' };
 
+class_type QVSubscription, { class => 'Net::Async::Webservice::UPS::QVSubscription' };
+
+class_type QVEvent, { class => 'Net::Async::Webservice::UPS::Response::QV::Event' };
+class_type QVFile, { class => 'Net::Async::Webservice::UPS::Response::QV::File' };
+class_type QVDelivery, { class => 'Net::Async::Webservice::UPS::Response::QV::Delivery' };
+class_type QVGeneric, { class => 'Net::Async::Webservice::UPS::Response::QV::Generic' };
+class_type QVOrigin, { class => 'Net::Async::Webservice::UPS::Response::QV::Origin' };
+class_type QVManifest, { class => 'Net::Async::Webservice::UPS::Response::QV::Manifest' };
+class_type QVPackage, { class => 'Net::Async::Webservice::UPS::Response::QV::Package' };
+class_type QVException, { class => 'Net::Async::Webservice::UPS::Response::QV::Exception' };
+class_type QVReference, { class => 'Net::Async::Webservice::UPS::Response::QV::Reference' };
+
 
 duck_type Cache, [qw(get set)];
 duck_type Cacheable, [qw(cache_id)];
@@ -305,7 +319,7 @@ Net::Async::Webservice::UPS::Types - type library for UPS
 
 =head1 VERSION
 
-version 1.0.7
+version 1.1.0
 
 =head1 DESCRIPTION
 
